// ScribDoc.cpp : implementation of the CScribbleDoc class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1995 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
#include "Scribble.h"

#include "ScribDoc.h"
#include "ScribItm.h"
#include "PenDlg.h"
#include "ScribVw.h"
#include "afxpriv.h"

#include "zip.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc

IMPLEMENT_DYNCREATE(CScribbleDoc, COleServerDoc)

BEGIN_MESSAGE_MAP(CScribbleDoc, COleServerDoc)
	//{{AFX_MSG_MAP(CScribbleDoc)
	ON_COMMAND(ID_EDIT_CLEAR_ALL, OnEditClearAll)
	ON_COMMAND(ID_PEN_THICK_OR_THIN, OnPenThickOrThin)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CLEAR_ALL, OnUpdateEditClearAll)
	ON_UPDATE_COMMAND_UI(ID_PEN_THICK_OR_THIN, OnUpdatePenThickOrThin)
	ON_COMMAND(ID_PEN_WIDTHS, OnPenWidths)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_FILE_SEND_ZIPPED_MAIL, OnFileSendZippedMail)
	ON_UPDATE_COMMAND_UI(ID_FILE_SEND_ZIPPED_MAIL, OnUpdateFileSendZippedMail)
	//}}AFX_MSG_MAP
#ifndef _MAC        
	ON_COMMAND(ID_FILE_SEND_MAIL, OnFileSendMail)
	ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL, OnUpdateFileSendMail)
#endif
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc construction/destruction

CScribbleDoc::CScribbleDoc()
{
	// Use OLE compound files
	EnableCompoundFile();

	m_sizeDoc = CSize(200, 200);
}

CScribbleDoc::~CScribbleDoc()
{
}

BOOL CScribbleDoc::OnNewDocument()
{
	if (!COleServerDoc::OnNewDocument())
		return FALSE;
	InitDocument();
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc serialization

void CScribbleDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		ar << m_sizeDoc;
	}
	else
	{
		ar >> m_sizeDoc;
	}
	m_strokeList.Serialize(ar);
}

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc diagnostics

#ifdef _DEBUG
void CScribbleDoc::AssertValid() const
{
	COleServerDoc::AssertValid();
}

void CScribbleDoc::Dump(CDumpContext& dc) const
{
	COleServerDoc::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc commands

BOOL CScribbleDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!COleServerDoc::OnOpenDocument(lpszPathName))
		return FALSE;
	InitDocument(); 
	return TRUE;
}

void CScribbleDoc::DeleteContents() 
{
	while (!m_strokeList.IsEmpty())
	{
		delete m_strokeList.RemoveHead();
	}
	COleServerDoc::DeleteContents();
}

void CScribbleDoc::InitDocument()
{
	m_bThickPen = FALSE;
	m_nThinWidth = 2;   // default thin pen is 2 pixels wide
	m_nThickWidth = 5;  // default thick pen is 5 pixels wide
	ReplacePen();       // initialize pen according to current width

	// default document size is 2 x 2 inches
	m_sizeDoc = CSize(200,200);
}

CStroke* CScribbleDoc::NewStroke()
{
	CStroke* pStrokeItem = new CStroke(m_nPenWidth);
	m_strokeList.AddTail(pStrokeItem);
	SetModifiedFlag();  // Mark the document as having been modified, for
						// purposes of confirming File Close.
	return pStrokeItem;
}




/////////////////////////////////////////////////////////////////////////////
// CStroke

IMPLEMENT_SERIAL(CStroke, CObject, 2)
CStroke::CStroke()
{
	// This empty constructor should be used by serialization only
}

CStroke::CStroke(UINT nPenWidth)
{
	m_nPenWidth = nPenWidth;
	m_rectBounding.SetRectEmpty();
}

void CStroke::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		ar << m_rectBounding;
		ar << (WORD)m_nPenWidth;
		m_pointArray.Serialize(ar);
	}
	else
	{
		ar >> m_rectBounding;
		WORD w;
		ar >> w;
		m_nPenWidth = w;
		m_pointArray.Serialize(ar);
	}
}

BOOL CStroke::DrawStroke(CDC* pDC)
{
	CPen penStroke;
	if (!penStroke.CreatePen(PS_SOLID, m_nPenWidth, RGB(0,0,0)))
		return FALSE;
	CPen* pOldPen = pDC->SelectObject(&penStroke);
	pDC->MoveTo(m_pointArray[0]);
	for (int i=1; i < m_pointArray.GetSize(); i++)
	{
		pDC->LineTo(m_pointArray[i]);
	}

	pDC->SelectObject(pOldPen);
	return TRUE;
}
void CScribbleDoc::OnEditClearAll() 
{
	DeleteContents();
	SetModifiedFlag();  // Mark the document as having been modified, for
						// purposes of confirming File Close.
	UpdateAllViews(NULL);
}

void CScribbleDoc::OnPenThickOrThin() 
{
	// Toggle the state of the pen between thin or thick.
	m_bThickPen = !m_bThickPen;

	// Change the current pen to reflect the new user-specified width.
	ReplacePen();
}

void CScribbleDoc::ReplacePen()
{
	m_nPenWidth = m_bThickPen? m_nThickWidth : m_nThinWidth;

	// Change the current pen to reflect the new user-specified width.
	m_penCur.DeleteObject();
	m_penCur.CreatePen(PS_SOLID, m_nPenWidth, RGB(0,0,0)); // solid black
}

void CScribbleDoc::OnUpdateEditClearAll(CCmdUI* pCmdUI) 
{
	// Enable the command user interface object (menu item or tool bar
	// button) if the document is non-empty, i.e., has at least one stroke.
	pCmdUI->Enable(!m_strokeList.IsEmpty());
}

void CScribbleDoc::OnUpdatePenThickOrThin(CCmdUI* pCmdUI) 
{
	// Add check mark to Draw Thick Line menu item, if the current
	// pen width is "thick".
	pCmdUI->SetCheck(m_bThickPen);
}

void CScribbleDoc::OnPenWidths() 
{
	CPenWidthsDlg dlg;
	// Initialize dialog data
	dlg.m_nThinWidth = m_nThinWidth;
	dlg.m_nThickWidth = m_nThickWidth;

	// Invoke the dialog box
	if (dlg.DoModal() == IDOK)
	{
		// retrieve the dialog data
		m_nThinWidth = dlg.m_nThinWidth;
		m_nThickWidth = dlg.m_nThickWidth;

		// Update the pen that is used by views when drawing new strokes,
		// to reflect the new pen width definitions for "thick" and "thin".
		ReplacePen();
	}
}

void CStroke::FinishStroke()
{
	// Calculate the bounding rectangle.  It's needed for smart
	// repainting.

	if (m_pointArray.GetSize()==0)
	{
		m_rectBounding.SetRectEmpty();
		return;
	}
	CPoint pt = m_pointArray[0];
	m_rectBounding = CRect(pt.x, pt.y, pt.x, pt.y);

	for (int i=1; i < m_pointArray.GetSize(); i++)
	{
		// If the point lies outside of the accumulated bounding
		// rectangle, then inflate the bounding rect to include it.
		pt = m_pointArray[i];
		m_rectBounding.left     = min(m_rectBounding.left, pt.x);
		m_rectBounding.right    = max(m_rectBounding.right, pt.x);
		m_rectBounding.top      = max(m_rectBounding.top, pt.y);
		m_rectBounding.bottom   = min(m_rectBounding.bottom, pt.y);
	}

	// Add the pen width to the bounding rectangle.  This is necessary
	// to account for the width of the stroke when invalidating
	// the screen.
	m_rectBounding.InflateRect(CSize(m_nPenWidth, -(int)m_nPenWidth));
	return;
}

COleServerItem* CScribbleDoc::OnGetEmbeddedItem()
{
	// OnGetEmbeddedItem is called by the framework to get the COleServerItem
	//  that is associated with the document.  It is only called when necessary.

	CScribbleItem* pItem = new CScribbleItem(this);
	ASSERT_VALID(pItem);
	return pItem;
}

void CScribbleDoc::OnSetItemRects(LPCRECT lpPosRect, LPCRECT lpClipRect)
{       
	// call base class to change the size of the window
	COleServerDoc::OnSetItemRects(lpPosRect, lpClipRect);

	// notify first view that scroll info should change
	POSITION pos = GetFirstViewPosition();
	CScribbleView* pView = (CScribbleView*)GetNextView(pos);
	pView->ResyncScrollSizes();
}

void CScribbleDoc::OnEditCopy()
{
	CScribbleItem* pItem = GetEmbeddedItem();
	pItem->CopyToClipboard(TRUE);
}



#ifndef _countof
#define _countof(array) (sizeof(array)/sizeof(array[0]))
#endif

uLong filetime(TCHAR *f, uLong * dt)
/*f: name of file to get info on */
/* dt: dostime */
{
    int ret = 0;  
    {      
        FILETIME ftLocal;      
        HANDLE hFind;
        WIN32_FIND_DATA  ff32;      
        hFind = FindFirstFile(f,&ff32);
        if (hFind != INVALID_HANDLE_VALUE)      
        {
            FileTimeToLocalFileTime(&(ff32.ftLastWriteTime),&ftLocal);
            FileTimeToDosDateTime(&ftLocal,((LPWORD)dt)+1,((LPWORD)dt)+0);
            FindClose(hFind);        
            ret = 1;      
        }  
    }  
    return ret;
}

#define WRITEBUFFERSIZE (16384)

void CScribbleDoc::OnFileSendZippedMail() 
{
    ASSERT_VALID(this);

    CString strMessage;
    CString strZipFile;
    int errclose = ZIP_OK;
    int err=0;
    BOOL bModified = IsModified(); //remember original modified flag.
    CString strOldName = m_strPathName;
    CString strDir;

#ifndef NO_COLESERVERDOC
    ASSERT(m_bRemember);

    LPSTORAGE lpOrigStg = m_lpRootStg;
    m_lpRootStg = NULL;
#endif
    TRY
    {
#ifndef NO_COLESERVERDOC
        m_bRemember = FALSE;
#endif
        CWaitCursor wait;
        TCHAR szTempName[_MAX_PATH];
        TCHAR szPath[_MAX_PATH];
        BOOL bRemoveTemp = FALSE;
        CString strFile;
        TCHAR szDrive[_MAX_DRIVE],szDir[_MAX_DIR], szName[_MAX_FNAME], szExt[_MAX_EXT];

	
        VERIFY(GetTempPath(_countof(szPath), szPath) != 0);
        if (m_strPathName.IsEmpty() || IsModified())
        {
            // save to temporary path.
            VERIFY(GetTempFileName(szPath, _T("afx"), 0, szTempName) != 0);

            BOOL bResult = DoSave(szTempName, FALSE);
		        
            if (!bResult)
            {
                //We could not save the temp file. Disk full? Not enough privileges?            
                AfxFormatString1(strMessage, AFX_IDP_FAILED_IO_ERROR_WRITE,szTempName);
                THROW (new CFileException);
            }
            bRemoveTemp = TRUE;
            strFile = m_strTitle;   
            if (m_strTitle.Find('.') == -1) // no extension.
            {
                CString strExt;
                CDocTemplate* pTemplate = GetDocTemplate();
                if (pTemplate != NULL && pTemplate->GetDocString(strExt, CDocTemplate::filterExt))
                {
                    strFile += strExt;
                }
            }
        }
        else
        {
            // use actual file since it isn't modified.
            lstrcpyn(szTempName, m_strPathName, _countof(szTempName));
            _tsplitpath(m_strPathName,NULL,NULL,szName, szExt);
            strFile = CString(szName)+ CString(szExt);
        }





        zipFile zf;
        //We try to create a temporary file in the temp-directory, delete it and create a directory with
        //the same name and use that for our newly created ZIP-File:
        VERIFY(GetTempFileName(szPath, _T("afx"), 0, strZipFile.GetBufferSetLength(_MAX_PATH+32) ) != 0);
        strZipFile.ReleaseBuffer();
        CFile::Remove(strZipFile);
        if (!CreateDirectory(strZipFile,NULL))
        {
            //Huh, our process has been preempted and someone else created a file or directory with our temp name?
            //Are we not allowed to create this directory? Not enough space to create this directory?
            //Get out of this hell!
            AfxFormatString1(strMessage, AFX_IDP_FAILED_ACCESS_WRITE,strZipFile);
            THROW (new CFileException);
        }
    

        strDir = strZipFile;  //save the temp directory in a variable so we can delete the directory afterwards
        strZipFile+=CString(_T("\\"))+strFile;
        _tsplitpath(strZipFile,szDrive,szDir,szName, szExt);

        _tmakepath(strZipFile.GetBufferSetLength(_MAX_PATH+32),szDrive,szDir,szName, _T("zip"));
        strZipFile.ReleaseBuffer();
#ifdef UNICODE
        USES_CONVERSION;
        zf = zipOpen(W2CA(strZipFile),0);
#else
        zf = zipOpen(strZipFile,0);
#endif

        FILE * fin;
        void* buf=NULL;
        int size_buf=0;
        size_buf = WRITEBUFFERSIZE;
        buf = (void*)malloc(size_buf);
        if (!buf)
        {
            //Not enough memory.
            strMessage.LoadString(AFX_IDS_MEMORY_EXCEPTION);
            THROW (new CMemoryException);
        }
        int size_read;
    
        zip_fileinfo zi;
        int opt_compress_level=Z_BEST_COMPRESSION;
        //Best compression to save bandwidth at a maximum.


        zi.tmz_date.tm_sec = zi.tmz_date.tm_min = zi.tmz_date.tm_hour = 
            zi.tmz_date.tm_mday = zi.tmz_date.tm_min = zi.tmz_date.tm_year = 0;
        zi.dosDate = 0;
        zi.internal_fa = 0;
        zi.external_fa = 0;
        filetime(szTempName,&zi.dosDate);

        err = zipOpenNewFileInZip(zf,
#ifdef UNICODE
        W2CA(strFile),
#else
        strFile,
#endif
        &zi,NULL,0,NULL,0,NULL /* comment*/,(opt_compress_level != 0) ? Z_DEFLATED : 0,opt_compress_level);

        ASSERT(err == ZIP_OK);
        if (err != ZIP_OK)
        {
            TRACE1("error in opening %s in zipfile\n",strFile);
            AfxFormatString1(strMessage, AFX_IDP_FAILED_IO_ERROR_WRITE,strFile);
            THROW (new CFileException); //if we would not return we would use an uninitialized fin variable
        }
        else
        {
            fin = _tfopen(szTempName,_T("rb"));
            if (fin==NULL)
            {
                err=ZIP_ERRNO;
                TRACE1("error in opening %s for reading\n",szTempName);
                strMessage.LoadString(AFX_IDP_FAILED_TO_OPEN_DOC);
                THROW (new CFileException); 
            }
        }

        do
        {
            err = ZIP_OK;
            size_read = fread(buf,1,size_buf,fin);
            if (size_read < size_buf)
                if (feof(fin)==0)
                {
                    //Seems like we could not read from the temp name.
                    AfxFormatString1(strMessage, AFX_IDP_FAILED_IO_ERROR_READ, szTempName);
                    AfxMessageBox(strMessage);
                    strMessage.Empty();
                    err = ZIP_ERRNO;
                }

            if (err==ZIP_OK && size_read>0)
            {
                err = zipWriteInFileInZip (zf,buf,size_read);
                if (err<0)
                {
                    //We could not write the file in the ZIP-File for whatever reason.
                    AfxFormatString1(strMessage, AFX_IDP_FAILED_IO_ERROR_WRITE,strZipFile);
                    AfxMessageBox(strMessage);
                    strMessage.Empty();
                }

            }
        }
        while (err==ZIP_OK && size_read>0);



        fclose(fin);
        errclose = zipClose(zf,NULL);
        if (bRemoveTemp)
            CFile::Remove(szTempName);
        free (buf);
    }
    CATCH_ALL(e)
    {
        SetModifiedFlag(bModified);
        m_strPathName = strOldName;
#ifndef NO_COLESERVERDOC
        m_lpRootStg = lpOrigStg;
        m_bRemember = TRUE;
#endif
        if (!strMessage.IsEmpty())
            {
            AfxMessageBox(strMessage);
            return;
            }
        else
            THROW_LAST();
    }
    END_CATCH_ALL

#ifndef NO_COLESERVERDOC
    m_lpRootStg = lpOrigStg;
    m_bRemember = TRUE;
#endif


    m_strPathName = strZipFile;
		
    if (errclose != ZIP_OK && err==ZIP_OK)
    {    
        //We could not close the ZIP-File
        AfxFormatString1(strMessage, AFX_IDP_FAILED_IO_ERROR_WRITE,strZipFile);
        AfxMessageBox(strMessage);
    }       
    else if (err==ZIP_OK) //if err!=ZIP_OK, something in creating the zip file went wrong, we already had a message box.
        OnFileSendMail(); //yes, everything went the right way up to now. Now invoke the MFC MAPI call OnFileSendMail().

    //After we have cheated the MFC OnFileSendMail() call with a ZIP-File instead of a real document, 
    //we can reset everything to the previously saved values:
    SetModifiedFlag(bModified);
    m_strPathName = strOldName;
    
    //From now on we don't care if calls succeed. Other apps may have our files or directories open, who knows?
    if (_tcslen(strZipFile))
        CFile::Remove(strZipFile);
    if (_tcslen(strDir))
        RemoveDirectory( strDir );        
}

void CScribbleDoc::OnUpdateFileSendZippedMail(CCmdUI* pCmdUI) 
{
	OnUpdateFileSendMail(pCmdUI);
}



